﻿
* Encoding: UTF-8.
*******************************************************************************************.
*  GULLIKSEN'S POOL     S P S S   C O D E   
*                                                                          
* AUTHORS:   URBANO LORENZO-SEVA & PERE J. FERRANDO
* URV, TARRAGONA (SPAIN)
* DATE: 29/SEPTEMBER/2022
*
*******************************************************************************************.
SET MXLOOP=30000.
SET MESSAGES=none.
set seed=random.
set length=none PRINTBACK = off WIDTH = 120.
matrix.

* UPDATE HERE THE NUMBER OF BOOTSTRAP SAMPLES (AT LEAST 500).

compute K = 3000.
 
* UPDATE HERE THE CONFIDENCE INTERVAL .

compute C = 0.95.
 
* UPDATE HERE THE MSA CRITERION TO REVISE ITEMS .
 
compute CRIT_MSA = 0.50.

* UPDATE HERE THE RDI CRITERIA TO INCLUDE ITEMS IN THE POOL (ALWAYS IN LINEAR PARAMETRIZATION).

compute spool_rdi_inf = 0.10.
compute spool_rdi_sup = 0.90.

* UPDATE HERE THE ICI CRITERIA TO INCLUDE ITEMS IN THE POOL (ALWAYS IN LINEAR PARAMETRIZATION).

compute spool_ici_inf = 0.20.
compute spool_ici_sup = 0.75.

* UPDATE HERE THE PARAMETRIZATION FOR THE OUTPUT.
* VALUE   1   LINEAR (INTERCEPT/CORRELATION).
* VALUE   2   NON LINEAR - ITEM RESPONSE THEORY (THRESHOLD/SLOPE).

compute parametrization = 2. 

*******************************************************************************************.
*##################  GULLIKSEN’S POOL CODE #######################.
do if K< 500.
     compute K = 3000.
end if.     

do if (C LT 0.50) OR (C GT 0.99).
     compute C = 0.95.
end if.     

do if (CRIT_MSA LT 0.30) OR (CRIT_MSA GT 0.70).
     compute CRIT_MSA = 0.50.
end if.     

get rawdata /var=all /names=vnames/missing = omit.
compute x=rawdata.
compute n=nrow(x).
compute m=ncol(x).
compute xmin = mmin(X).
compute xmax = mmax(X).
compute RDI = T((csum(X)/n -xmin)/(xmax-xmin)).

compute ones=make(n,1,1).
compute mx=csum(x)/n.
compute variance=(csum( (x-ones*mx) &* (x-ones*mx ) )/(n-1)).
compute sx=sqrt(variance).
compute ddata=x-ones*mx.
compute dSx=make(m,m,0).
call setdiag(dSx,sx).
compute R = inv(dSx)*(t(ddata)*ddata/(n-1))*inv(dSx).
compute IR = inv(R).
compute SS=make(m,m,0).
compute tmp = diag(IR).
call setdiag(SS,tmp).
compute SS = inv(sqrt(SS)).
compute Q = SS * IR * SS.
compute RS = R &* R.
compute QS = Q &* Q.
loop #ii= 1 to m.
        compute RS(#ii,#ii)=0.
        compute QS(#ii,#ii)=0.
end loop.
compute MNUM = t(csum(RS)).
compute MDEN = t(csum(RS+QS)).
compute MSA = MNUM &/ MDEN.

compute ICI = make(m,1,0).
compute RE = make(m,1,0).
compute deno = spool_ici_sup * spool_ici_sup * 0.5 * 0.5.
loop #i=1 to m.
     compute ICI(#i,1) = sqrt(1 - 1/IR(#i,#i)).
    compute RE(#i,1) = ICI(#i,1)*ICI(#i,1)*RDI(#i,1)*(1-RDI(#i,1))/ deno.
   do if RE(#i,1) GT 1.
        compute RE(#i,1)=1.
   end if.
end loop.

compute MSAB=make(K,m,0).
compute RDIB=make(K,m,0).
compute ICIB=make(K,m,0).
compute REB=make(K,m,0).
loop #i= 1 to K.
      compute ind = 1+trunc(n*uniform(n,1)).
      compute xi = x(ind,:).
      compute mx=csum(xi)/n. 
      compute variance=(csum( (xi-ones*mx) &* (xi-ones*mx ) )/(n-1)).
      compute sx=sqrt(variance).
      compute ddata=xi-ones*mx.
      compute dSx=make(m,m,0).
      call setdiag(dSx,sx).
      compute Ri = inv(dSx)*(t(ddata)*ddata/(n-1))*inv(dSx).
      compute IR = inv(Ri).
      compute SS=make(m,m,0).
      compute tmp = diag(IR).
      call setdiag(SS,tmp).
      compute SS = inv(sqrt(SS)).
      compute Q = SS * IR * SS.
      compute RS = Ri &* Ri.
      compute QS = Q &* Q.
      loop #ii= 1 to m.
           compute RS(#ii,#ii)=0.
           compute QS(#ii,#ii)=0.
      end loop.
      compute MNUM = csum(RS).
      compute MDEN = csum(RS+QS).
      compute MSAi = MNUM &/ MDEN.
      compute MSAB(#i,:)=MSAi.
      compute RDIB(#i,:) = (csum(xi)/n -xmin)/(xmax-xmin).
      loop #j=1 to m.
         compute ICIB(#i,#j) = sqrt(1 - 1/IR(#j,#j)).
         compute REB(#i,#j) = ICIB(#i,#j)*ICIB(#i,#j)*RDIB(#i,#j)*(1-RDIB(#i,#j))/ deno.
         do if REB(#i,#j) GT 1.
              compute REB(#i,#j)=1.
         end if.
      end loop.
end loop.
compute MSAB_IC = make(m,2,0).
compute RDIB_IC = make(m,2,0).
compute ICIB_IC = make(m,2,0).
compute REB_IC = make(m,2,0).
compute in_msa = make(m,1,1).
compute in_rdi= make(m,1,1).
compute in_ici = make(m,1,1).
loop #i=1 to m.
       compute s = MSAB(:,#i).
       compute s(grade(MSAB(:,#i))) = MSAB(:,#i).
       compute MSAB_IC(#i,1) = s(1+trunc(K*((1-C)/2))).
       compute MSAB_IC(#i,2) = s(1+trunc(K*((1+C)/2))).
       compute in_msa(#i,1)= MSAB_IC(#i,1) GT CRIT_MSA.

       compute s = RDIB(:,#i).
       compute s(grade(RDIB(:,#i))) = RDIB(:,#i).
       compute RDIB_IC(#i,1) = s(1+trunc(K*((1-C)/2))).
       compute RDIB_IC(#i,2) = s(1+trunc(K*((1+C)/2))).
       compute in_rdi(#i,1)= ((1-(RDIB_IC(#i,2) LT spool_rdi_inf)) * (1-(RDIB_IC(#i,1) GT spool_rdi_sup))).

       compute s = ICIB(:,#i).
       compute s(grade(ICIB(:,#i))) = ICIB(:,#i).
       compute ICIB_IC(#i,1) = s(1+trunc(K*((1-C)/2))).
       compute ICIB_IC(#i,2) = s(1+trunc(K*((1+C)/2))).
       compute in_ici(#i,1)= ((1-(ICIB_IC(#i,2) LT spool_ici_inf)) * (1-(ICIB_IC(#i,1) GT spool_ici_sup))).

       compute s = REB(:,#i).
       compute s(grade(REB(:,#i))) = REB(:,#i).
       compute REB_IC(#i,1) = s(1+trunc(K*((1-C)/2))).
       compute REB_IC(#i,2) = s(1+trunc(K*((1+C)/2))).
end loop.
compute revise = m-csum(in_msa &* in_ici &* in_rdi).

compute n_pool_in = csum(in_ici &* in_rdi).
compute pool_in = make(n_pool_in,6,0).
compute pool_out = make(m-n_pool_in,6,0).
compute pool_in_na = vnames.
compute pool_out_na = vnames.
compute msa_in = make(m-revise,3,0).
compute msa_out = make(revise,3,0).
compute re_in = make(m-revise,3,0).
compute msa_in_na = vnames.
compute msa_out_na = vnames.
compute #j=1.
compute #jj=1.
compute #h=1.
compute #hh=1.
compute dummy = in_ici &* in_rdi.
compute dummy2 = in_msa &* in_ici &* in_rdi.
do if parametrization EQ 2.
    loop #i=1 to m.
        compute RDI(#i,1) = -1 * IDF.NORMAL(RDI(#i,1), 0, 1).
        compute RDIB_IC(#i,1) = -1 * IDF.NORMAL(RDIB_IC(#i,1), 0, 1).
        compute RDIB_IC(#i,2) = -1 * IDF.NORMAL(RDIB_IC(#i,2), 0, 1).
        compute ICI(#i,1) = ICI(#i,1)/sqrt(1 - (ICI(#i,1)*ICI(#i,1))).
        compute ICIB_IC(#i,1) = ICIB_IC(#i,1)/sqrt(1 - (ICIB_IC(#i,1)*ICIB_IC(#i,1))).
        compute ICIB_IC(#i,2) = ICIB_IC(#i,2)/sqrt(1 - (ICIB_IC(#i,2)*ICIB_IC(#i,2))).
    end loop.
end if.
loop #i=1 to m.
       do if dummy(#i) EQ 1.
            compute pool_in_na(#j) = vnames(#i).
            compute pool_in(#j,1) = RDI(#i,1).
            compute pool_in(#j,2) = RDIB_IC(#i,1).
            compute pool_in(#j,3) = RDIB_IC(#i,2).
            compute pool_in(#j,4) = ICI(#i,1).
            compute pool_in(#j,5) = ICIB_IC(#i,1).
            compute pool_in(#j,6) = ICIB_IC(#i,2).
            compute #j = #j+1.
       end if.
       do if dummy(#i) EQ 0.
            compute pool_out_na(#jj) = vnames(#i).
            compute pool_out(#jj,1) = RDI(#i,1).
            compute pool_out(#jj,2) = RDIB_IC(#i,1).
            compute pool_out(#jj,3) = RDIB_IC(#i,2).
            compute pool_out(#jj,4) = ICI(#i,1).
            compute pool_out(#jj,5) = ICIB_IC(#i,1).
            compute pool_out(#jj,6) = ICIB_IC(#i,2).
            compute #jj = #jj+1.
       end if.
       do if dummy2(#i) EQ 1.
            compute msa_in_na(#j) = vnames(#i).
            compute msa_in(#h,1) = MSA(#i,1).
            compute msa_in(#h,2) = MSAB_IC(#i,1).
            compute msa_in(#h,3) = MSAB_IC(#i,2).
            compute re_in(#h,1) = RE(#i,1).
            compute re_in(#h,2) = REB_IC(#i,1).
            compute re_in(#h,3) = REB_IC(#i,2).
            compute #h = #h+1.
       end if.
       do if dummy2(#i) EQ 0.
            compute msa_out_na(#j) = vnames(#i).
            compute msa_out(#hh,1) = MSA(#i,1).
            compute msa_out(#hh,2) = MSAB_IC(#i,1).
            compute msa_out(#hh,3) = MSAB_IC(#i,2).
            compute #hh = #hh+1.
       end if.
end loop.

print / space = 2.
print /title="*************************************************************************************************".
print / space = 1.
print /title="                    GULLIKSEN'S POOL     S P S S   C O D E                    ".
print / space = 2.
print /title="          Authors: Urbano Lorenzo-Seva   &   Pere J. Ferrando                     ".
print / space = 2.
print /title="               Universitat Rovira i Virgili, Tarragona, Spain              ".
print / space = 1.
print /title="                         Version: 2.01, September 2022                          ".
print / space = 2.
print /title="*************************************************************************************************".
print / space = 2.
do if parametrization  EQ 1.
    print /title= 'Parametrization: LINEAR (INTERCEPT/CORRELATION)'.
end if.
do if parametrization  EQ 2.
    print /title= 'Parametrization: NON LINEAR - ITEM RESPONSE THEORY (THRESHOLD/SLOPE) '.
end if.
print / space = 2.
print /title= 'Number of items proposed to be revised'.
print {revise}.

do if parametrization  EQ 1.
	print /title="*************************************************************************************************".
	print /title="STEP 1: The pool based on Relative Difficulty Index (RDI) and Item Consistency Index (ICI)".
	print /title="*************************************************************************************************".
	print / space = 1.
	print {pool_in}
          /format='F8.3'
          /rname = pool_in_na
          /clabel = 'RDI', 'Lower','Upper','ICI', 'Lower','Upper'
          /title='Items admited in the pool in the first step'.
	print {pool_out}
          /format='F8.3'
          /rname = pool_out_na
          /clabel = 'RDI', 'Lower','Upper','ICI', 'Lower','Upper'
          /title='Items that might not work - Revise'.
end if.
do if parametrization  EQ 2.
	print /title="*************************************************************************************************".
	print /title="STEP 1: The pool based on Overall Item Threshold (OIT) and Overall Item Slope (OIS) ".
	print /title="*************************************************************************************************".
	print / space = 1.
	print {pool_in}
          /format='F8.3'
          /rname = pool_in_na
          /clabel = 'OIT', 'Lower','Upper','OIS', 'Lower','Upper'
          /title='Items admited in the pool in the first step'.
	print {pool_out}
          /format='F8.3'
          /rname = pool_out_na
          /clabel = 'OIT', 'Lower','Upper','OIS', 'Lower','Upper'
          /title='Items that might not work - Revise'.
end if.
print / space = 2.
print /title="*************************************************************************************************".
print /title="STEP 2: Assessment of the measure of sampling adequacy (MSA) at the item level    ".
print /title="*************************************************************************************************".
print / space = 1.
print {msa_in}
          /format='F8.3'
          /rname = msa_in_na
          /clabel = 'MSA', 'Lower','Upper','ICI', 'Lower','Upper'
          /title='Items admited in the pool in the second step'.
print {msa_out}
          /format='F8.3'
          /rname = msa_in_na
          /clabel = 'MSA', 'Lower','Upper','ICI', 'Lower','Upper'
          /title='Items that might not work - Revise'.
print / space = 2.
print /title="*************************************************************************************************".
print /title="STEP 3: Inspection of the Relative Efficiency for the items accepted in the pool           ".
print /title="*************************************************************************************************".
print / space = 1.
print {re_in}
          /format='F8.3'
          /rname = msa_in_na
          /clabel = 'RE', 'Lower','Upper','ICI', 'Lower','Upper'
          /title='Final set of items included in the pool'.

end matrix.
*##################  END OF GULLIKSEN’S POOL CODE #######################.

